import yaml
import sys
import pprint
import math

def check_bool(value, expected):
    if expected == 'false()' and value is False:
        return 1
    if expected == 'true()' and value is True:
        return 1
    print(value)
    print(expected)
    return 0

def check_int(value, expected):
    if (int(expected) == value):
        return 1
    print(value)
    print(expected)
    return 0

def check_float(value, expected):
    if expected == 'inf()':
        if value == math.inf:
            return 1
    elif expected == 'inf-neg()':
        if value == -math.inf:
            return 1
    elif expected == 'nan()':
        if math.isnan(value):
            return 1
    elif (float(expected) == value):
        return 1
    else:
        print(value)
        print(expected)
        return 0

def check_str(value, expected):
    if value == expected:
        return 1
    print(value)
    print(expected)
    return 0


def _fail(input, test):
    print("Input: >>" + input + "<<")
    print(test)

# The tests/data/yaml11.schema file is copied from
# https://github.com/perlpunk/yaml-test-schema/blob/master/data/schema-yaml11.yaml
def test_implicit_resolver(data_filename, skip_filename, verbose=False):
    types = {
        'str':   [str,   check_str],
        'int':   [int,   check_int],
        'float': [float, check_float],
        'inf':   [float, check_float],
        'nan':   [float, check_float],
        'bool':  [bool,  check_bool],
    }
    with open(skip_filename, 'rb') as file:
        skipdata = yaml.load(file, Loader=yaml.SafeLoader)
    skip_load = skipdata['load']
    skip_dump = skipdata['dump']
    if verbose:
        print(skip_load)
    with open(data_filename, 'rb') as file:
        tests = yaml.load(file, Loader=yaml.SafeLoader)

    i = 0
    fail = 0
    for i, (input, test) in enumerate(sorted(tests.items())):
        if verbose:
            print('-------------------- ' + str(i))

        # Skip known loader bugs
        if input in skip_load:
            continue

        exp_type = test[0]
        data     = test[1]
        exp_dump = test[2]

        # Test loading
        try:
            loaded = yaml.safe_load(input)
        except:
            print("Error:", sys.exc_info()[0], '(', sys.exc_info()[1], ')')
            fail+=1
            _fail(input, test)
            continue

        if verbose:
            print(input)
            print(test)
            print(loaded)
            print(type(loaded))

        if exp_type == 'null':
            if loaded is None:
                pass
            else:
                fail+=1
                _fail(input, test)
        else:
            t = types[exp_type][0]
            code = types[exp_type][1]
            if isinstance(loaded, t):
                if code(loaded, data):
                    pass
                else:
                    fail+=1
                    _fail(input, test)
            else:
                fail+=1
                _fail(input, test)

        # Skip known dumper bugs
        if input in skip_dump:
            continue

        dump = yaml.safe_dump(loaded, explicit_end=False)
        # strip trailing newlines and footers
        if dump.endswith('\n...\n'):
            dump = dump[:-5]
        if dump.endswith('\n'):
            dump = dump[:-1]
        if dump == exp_dump:
            pass
        else:
            print("Compare: >>" + dump + "<< >>" + exp_dump + "<<")
            fail+=1
            _fail(input, test)

#        if i >= 80:
#            break

    if fail > 0:
        print("Failed " + str(fail) + " / " + str(i) + " tests")
        assert(False)
    else:
        print("Passed " + str(i) + " tests")
    print("Skipped " + str(len(skip_load)) + " load tests")
    print("Skipped " + str(len(skip_dump)) + " dump tests")

test_implicit_resolver.unittest = ['.schema', '.schema-skip']

if __name__ == '__main__':
    import test_appliance
    test_appliance.run(globals())




import yaml
import pprint
import sys

def test_sort_keys(input_filename, sorted_filename, verbose=False):
    with open(input_filename, 'rb') as file:
        input = file.read().decode('utf-8')
    with open(sorted_filename, 'rb') as file:
        sorted = file.read().decode('utf-8')
    data = yaml.load(input, Loader=yaml.FullLoader)
    dump_sorted = yaml.dump(data, default_flow_style=False, sort_keys=True)
    dump_unsorted = yaml.dump(data, default_flow_style=False, sort_keys=False)
    dump_unsorted_safe = yaml.dump(data, default_flow_style=False, sort_keys=False, Dumper=yaml.SafeDumper)
    if verbose:
        print("INPUT:")
        print(input)
        print("DATA:")
        print(data)

    assert dump_sorted == sorted

    if sys.version_info>=(3,7):
        assert dump_unsorted == input
        assert dump_unsorted_safe == input

test_sort_keys.unittest = ['.sort', '.sorted']

if __name__ == '__main__':
    import test_appliance
    test_appliance.run(globals())




import yaml

def test_dump(verbose=False):
    assert yaml.dump(['foo'])
test_dump.unittest = True

def test_load_no_loader(verbose=False):
    try:
        yaml.load("- foo\n")
    except TypeError:
        return True
    assert(False, "load() require Loader=...")
test_load_no_loader.unittest = True

def test_load_safeloader(verbose=False):
    assert yaml.load("- foo\n", Loader=yaml.SafeLoader)
test_load_safeloader.unittest = True

if __name__ == '__main__':
    import sys, test_load
    sys.modules['test_load'] = sys.modules['__main__']
    import test_appliance
    test_appliance.run(globals())
